@@ -1,5 +1,5 @@
-/*      Orage - Calendar and alarm handler
- *
+/*
+ * Copyright (c) 2021-2023 Erkki Moorits
  * Copyright (c) 2005-2013 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2004-2005 Mickael Graf (korbinus at xfce.org)
  *
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the 
-       Free Software Foundation
-       51 Franklin Street, 5th Floor
-       Boston, MA 02110-1301 USA
-
+ * along with this program; if not, write to the
+ *     Free Software Foundation
+ *     51 Franklin Street, 5th Floor
+ *     Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,16 +51,16 @@
 #include <glib.h>
 #include <glib/gprintf.h>
 
+#include "orage-appointment-window.h"
+#include "orage-category.h"
 #include "orage-i18n.h"
+#include "orage-window.h"
 #include "functions.h"
-#include "mainbox.h"
 #include "reminder.h"
-#include "about-xfcalendar.h"
 #include "ical-code.h"
 #include "event-list.h"
-#include "appointment.h"
 #include "parameters.h"
-#include "day-view.h"
+#include "orage-week-window.h"
 
 #ifdef HAVE_X11_TRAY_ICON
 #include "tray_icon.h"
@@ -89,24 +88,10 @@ static const GtkTargetEntry drag_targets[] =
     { "STRING", 0, DRAG_TARGET_STRING }
 };
 
-static void do_appt_win (const appt_win_action mode, char *uid, el_win *el,
-                         GDateTime *gdt)
-{
-    appt_win *apptw;
-
-    apptw = create_appt_win(mode, uid, gdt);
-    if (apptw) {
-        /* we started this, so keep track of it */
-        el->apptw_list = g_list_prepend(el->apptw_list, apptw);
-        /* inform the appointment that we are interested in it */
-        apptw->el = el;
-    }
-}
-
-static void start_appt_win (const appt_win_action mode,  el_win *el
+static void start_appt_win (const gboolean copy_appt_win, el_win *el
         , GtkTreeModel *model, GtkTreeIter *iter, GtkTreePath *path)
 {
-    GDateTime *gdt;
+    GtkWidget *apptw;
     gchar *uid = NULL, *flags = NULL;
 
     if (gtk_tree_model_get_iter(model, iter, path)) {
@@ -121,10 +106,21 @@ static void start_appt_win (const appt_win_action mode,  el_win *el
         }
         g_free(flags);
 #endif
-        gdt = g_date_time_new_now_local ();
-        do_appt_win(mode, uid, el, gdt);
-        g_date_time_unref (gdt);
+        if (copy_appt_win)
+            apptw = orage_appointment_window_new_copy (uid);
+        else
+            apptw = orage_appointment_window_new_update (uid);
+
         g_free(uid);
+
+        /* inform the appointment that we are interested in it */
+        orage_appointment_window_set_event_list (
+                ORAGE_APPOINTMENT_WINDOW (apptw), el);
+
+        gtk_window_present (GTK_WINDOW (apptw));
+
+        /* we started this, so keep track of it */
+        el->apptw_list = g_list_prepend (el->apptw_list, apptw);
     }
 }
 
@@ -136,7 +132,7 @@ static void editEvent(GtkTreeView *view, GtkTreePath *path
     GtkTreeIter   iter;
 
     model = gtk_tree_view_get_model(view);
-    start_appt_win (UPDATE_APPT_WIN, el, model, &iter, path);
+    start_appt_win (FALSE, el, model, &iter, path);
 }
 
 static gint sortEvent_comp(GtkTreeModel *model
@@ -683,7 +679,7 @@ static void journal_data(el_win *el)
 
 void refresh_el_win(el_win *el)
 {
-    orage_category_get_list();
+    (void)orage_category_get_list (); /* Uses side effects. */
     if (el->Window && el->ListStore && el->TreeView) {
         gtk_list_store_clear(el->ListStore);
         el->page = gtk_notebook_get_current_page(GTK_NOTEBOOK(el->Notebook));
@@ -774,9 +770,12 @@ static void set_el_data (el_win *el, GDateTime *gdt)
 static void set_el_data_from_cal(el_win *el)
 {
     GDateTime *gdt;
+    OrageApplication *app;
+    OrageWindow *window;
 
-    gdt = orage_cal_to_gdatetime (
-            GTK_CALENDAR(((CalWin *)g_par.xfcal)->mCalendar), 0, 0);
+    app = ORAGE_APPLICATION (g_application_get_default ());
+    window = ORAGE_WINDOW (orage_application_get_window (app));
+    gdt = orage_cal_to_gdatetime (orage_window_get_calendar (window), 0, 0);
     set_el_data (el, gdt);
     g_date_time_unref (gdt);
 }
@@ -797,7 +796,7 @@ static void duplicate_appointment(el_win *el)
         if (list_len > 1)
             g_warning ("Copy: too many rows selected");
         path = (GtkTreePath *)g_list_nth_data(list, 0);
-        start_appt_win (COPY_APPT_WIN, el, model, &iter, path);
+        start_appt_win (TRUE, el, model, &iter, path);
     }
     else {
         orage_info_dialog(GTK_WINDOW(el->Window)
@@ -823,7 +822,7 @@ static void on_File_duplicate_activate_cb (G_GNUC_UNUSED GtkMenuItem *mi,
 
 static void close_window(el_win *el)
 {
-    appt_win *apptw;
+    OrageAppointmentWindow *apptw;
     GList *apptw_list;
 
     gtk_window_get_size(GTK_WINDOW(el->Window)
@@ -838,9 +837,12 @@ static void close_window(el_win *el)
     for (apptw_list = g_list_first(apptw_list);
          apptw_list != NULL;
          apptw_list = g_list_next(apptw_list)) {
-        apptw = (appt_win *)apptw_list->data;
+        apptw = ORAGE_APPOINTMENT_WINDOW (apptw_list->data);
         if (apptw) /* appointment window is still alive */
-            apptw->el = NULL; /* not interested anymore */
+        {
+            /* not interested anymore */
+            orage_appointment_window_set_event_list  (apptw, NULL);
+        }
         else
             g_warning ("%s: not null appt window", G_STRFUNC);
     }
@@ -862,7 +864,7 @@ static void on_Dayview_clicked (G_GNUC_UNUSED GtkButton *b, gpointer user_data)
     GDateTime *gdt;
 
     gdt = g_object_get_data (G_OBJECT (el->Window), DATE_KEY);
-    create_day_win (gdt);
+    orage_week_window_build (gdt);
 }
 
 static void on_File_close_activate_cb (G_GNUC_UNUSED GtkMenuItem *mi,
@@ -895,10 +897,14 @@ static void changeSelectedDate (el_win *el, const gint day)
 {
     GDateTime *gdt1;
     GDateTime *gdt2;
+    OrageWindow *window;
+    OrageApplication *app;
 
     gdt1 = g_object_get_data (G_OBJECT (el->Window), DATE_KEY);
     gdt2 = g_date_time_add_days (gdt1, day);
-    orage_select_date (GTK_CALENDAR(((CalWin *)g_par.xfcal)->mCalendar), gdt2);
+    app = ORAGE_APPLICATION (g_application_get_default ());
+    window = ORAGE_WINDOW (orage_application_get_window (app));
+    orage_select_date (orage_window_get_calendar (window), gdt2);
     g_date_time_unref (gdt2);
 
     set_el_data_from_cal(el);
@@ -917,7 +923,9 @@ static void on_Go_previous_activate_cb (G_GNUC_UNUSED GtkMenuItem *mi,
 
 static void go_to_today(el_win *el)
 {
-    orage_select_today(GTK_CALENDAR(((CalWin *)g_par.xfcal)->mCalendar));
+    OrageApplication *app = ORAGE_APPLICATION (g_application_get_default ());
+    OrageWindow *window = ORAGE_WINDOW (orage_application_get_window (app));
+    orage_select_today (orage_window_get_calendar (window));
     set_el_data_from_cal(el);
 }
 
@@ -946,9 +954,16 @@ static void on_Go_next_activate_cb (G_GNUC_UNUSED GtkMenuItem *mi,
 static void create_new_appointment(el_win *el)
 {
     GDateTime *gdt;
+    GtkWidget *appointment_window;
 
     gdt = g_object_get_data (G_OBJECT (el->Window), DATE_KEY);
-    do_appt_win (NEW_APPT_WIN, NULL, el, gdt);
+    appointment_window = orage_appointment_window_new (gdt);
+    /* inform the appointment that we are interested in it */
+    orage_appointment_window_set_event_list (
+            ORAGE_APPOINTMENT_WINDOW (appointment_window), el);
+    gtk_window_present (GTK_WINDOW (appointment_window));
+    /* we started this, so keep track of it */
+    el->apptw_list = g_list_prepend (el->apptw_list, appointment_window);
 }
 
 static void on_File_newApp_activate_cb (G_GNUC_UNUSED GtkMenuItem *mi,
@@ -987,6 +1002,7 @@ static void on_show_old_clicked (G_GNUC_UNUSED GtkCheckButton *b,
 
 static void delete_appointment(el_win *el)
 {
+    OrageApplication *app;
     gint result;
     GtkTreeSelection *sel;
     GtkTreeModel     *model;
@@ -1019,7 +1035,7 @@ static void delete_appointment(el_win *el)
                     /* note that file id changes after archive */ 
                     uid[0]='O';
                     /* xfical_unarchive_uid closes the file */
-                    if (!xfical_file_open(TRUE)) 
+                    if (!xfical_file_open(TRUE))
                         return;
                 }
                 g_free(flags);
@@ -1034,7 +1050,8 @@ static void delete_appointment(el_win *el)
         }
         xfical_file_close(TRUE);
         refresh_el_win(el);
-        orage_mark_appointments();
+        app = ORAGE_APPLICATION (g_application_get_default ());
+        orage_mark_appointments (ORAGE_WINDOW (orage_application_get_window (app)));
         g_list_foreach(list, (GFunc)(GCallback)gtk_tree_path_free, NULL);
         g_list_free(list);
     }
@@ -1297,7 +1314,7 @@ static void build_event_tab(el_win *el)
                                                   "margin-left", 15,
                                                   NULL);
     gtk_widget_set_tooltip_text(el->event_show_old_checkbutton
-            , _("Check this if you want to see old events also. This can only be selected after 'only first repeating' is enabled to avoid very long lists.\nNote that extra days selection still defines if newer appointments are listed'."));
+            , _("Check this if you want to see old events also. This can only be selected after 'only first repeating' is enabled to avoid very long lists.\nNote that extra days selection still defines if newer appointments are listed."));
     gtk_toggle_button_set_active(
             GTK_TOGGLE_BUTTON(el->event_show_old_checkbutton), el->only_first);
     gtk_widget_set_sensitive(el->event_show_old_checkbutton, el->only_first);
