@@ -1,5 +1,5 @@
-/*      Orage - Calendar and alarm handler
- *
+/*
+ * Copyright (c) 2021-2023 Erkki Moorits
  * Copyright (c) 2005-2011 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2004-2005 Mickael Graf (korbinus at xfce.org)
  *
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the 
-       Free Software Foundation
-       51 Franklin Street, 5th Floor
-       Boston, MA 02110-1301 USA
-
+ * along with this program; if not, write to the
+ *     Free Software Foundation
+ *     51 Franklin Street, 5th Floor
+ *     Boston, MA 02110-1301 USA
  */
 
 #ifndef __ICAL_CODE_H__
@@ -26,7 +25,7 @@
 
 #include "ical-expimp.h"
 
-#define XFICAL_UID_LEN 200
+#define MAX_MONTH_RECURRENCE_DAY 28
 
 typedef enum
 {
@@ -45,6 +44,56 @@ typedef enum
    ,XFICAL_TYPE_JOURNAL
 } xfical_type;
 
+typedef enum
+{
+    XFICAL_RECUR_NO_LIMIT = 0,
+    XFICAL_RECUR_COUNT,
+    XFICAL_RECUR_UNTIL
+} xfical_recur_limit;
+
+typedef enum
+{
+    XFICAL_RECUR_MONTH_TYPE_BEGIN = 0,
+    XFICAL_RECUR_MONTH_TYPE_END,
+    XFICAL_RECUR_MONTH_TYPE_EVERY
+} xfical_recur_month_type;
+
+typedef enum
+{
+    XFICAL_RECUR_MONTH_DAY_MON = 0,
+    XFICAL_RECUR_MONTH_DAY_TUE,
+    XFICAL_RECUR_MONTH_DAY_WED,
+    XFICAL_RECUR_MONTH_DAY_THU,
+    XFICAL_RECUR_MONTH_DAY_FRI,
+    XFICAL_RECUR_MONTH_DAY_SAT,
+    XFICAL_RECUR_MONTH_DAY_SUN
+} xfical_recur_day_sel;
+
+typedef enum
+{
+    XFICAL_RECUR_MONTH_WEEK_FIRST = 0,
+    XFICAL_RECUR_MONTH_WEEK_SECOND,
+    XFICAL_RECUR_MONTH_WEEK_THIRD,
+    XFICAL_RECUR_MONTH_WEEK_FOURTH,
+    XFICAL_RECUR_MONTH_WEEK_LAST
+} xfical_recur_week_sel;
+
+typedef enum
+{
+    XFICAL_RECUR_MONTH_JAN = 0,
+    XFICAL_RECUR_MONTH_FEB,
+    XFICAL_RECUR_MONTH_MAR,
+    XFICAL_RECUR_MONTH_APR,
+    XFICAL_RECUR_MONTH_MAY,
+    XFICAL_RECUR_MONTH_JUN,
+    XFICAL_RECUR_MONTH_JUL,
+    XFICAL_RECUR_MONTH_AUG,
+    XFICAL_RECUR_MONTH_SEP,
+    XFICAL_RECUR_MONTH_OCT,
+    XFICAL_RECUR_MONTH_NOV,
+    XFICAL_RECUR_MONTH_DEC
+} xfical_recur_month_sel;
+
 typedef struct _xfical_appt
 {
     xfical_type type;
@@ -114,12 +163,16 @@ typedef struct _xfical_appt
     GDateTime *starttimecur;
     GDateTime *endtimecur;
     xfical_freq freq;
-    gint   recur_limit; /* 0 = no limit  1 = count  2 = until */
-    gint   recur_count;
+    xfical_recur_limit recur_limit;
+    gint recur_count;
+    xfical_recur_month_type recur_month_type;
+    guint recur_month_days;
+    xfical_recur_week_sel recur_week_sel;
+    xfical_recur_day_sel recur_day_sel;
+    xfical_recur_month_sel recur_month_sel;
 
     GDateTime *recur_until;
     gboolean recur_byday[7]; /* 0=Mo, 1=Tu, 2=We, 3=Th, 4=Fr, 5=Sa, 6=Su */
-    gint   recur_byday_cnt[7]; /* monthly/early: 1=first -1=last 2=second... */
     gint   interval;    /* 1=every day/week..., 2=every second day/week,... */
     gboolean recur_todo_base_start; /* TRUE=start time, FALSE=completed time */
     GList  *recur_exceptions; /* EXDATE and RDATE list xfical_exception */
@@ -131,12 +184,49 @@ gboolean xfical_file_open(gboolean foreign);
 void xfical_file_close(gboolean foreign);
 void xfical_file_close_force(void);
 
-xfical_appt *xfical_appt_alloc (void);
-char *xfical_appt_add(char *ical_file_id, xfical_appt *appt);
+/** Allocates memory and initializes for given data it for new ical_type
+ *  structure.
+ *  @param gdt date for initialization
+ *  @return NULL if failed and pointer to xfical_appt if successfull. You must
+ *          free it after not being used anymore. (g_free())
+ */
+xfical_appt *xfical_appt_new_day (GDateTime *gdt);
+
+/** Add EVENT/TODO/JOURNAL type ical appointment to ical file
+ *  @param appt pointer to filled xfical_appt structure, which is to be stored.
+ *              Caller is responsible for filling and allocating and freeing it.
+ *  @returns NULL if failed and new ical id if successfully added. This ical id
+ *           is owned by the routine. Do not deallocate it. It will be
+ *           overwrittewritten by next invocation of this function.
+ */
+gchar *xfical_appt_add (gchar *ical_file_id, xfical_appt *appt);
+
+/** Read EVENT from ical datafile.
+ *  @param ical_uid key of ical EVENT appt-> is to be read
+ *  @return if failed: NULL
+ *          if successfull: xfical_appt pointer to xfical_appt struct filled
+ *                          with data. You must free it after not being used
+ *                          anymore using xfical_appt_free.
+ *
+ *  @note This routine does not fill starttimecur nor endtimecur, those are
+ *  always initialized to null string
+ */
 xfical_appt *xfical_appt_get(const gchar *ical_id);
 void xfical_appt_free(xfical_appt *appt);
-gboolean xfical_appt_mod(char *ical_id, xfical_appt *appt);
-gboolean xfical_appt_del(char *ical_id);
+
+ /** Modify EVENT type ical appointment in ical file.
+  *  @param ical_uid char pointer to ical ical_uid to modify
+  *  @param app pointer to filled xfical_appt structure, which is stored. Caller
+  *         is responsible for filling and allocating and freeing it.
+  *  @return TRUE is successfull, FALSE if failed
+  */
+gboolean xfical_appt_mod (gchar *ical_id, xfical_appt *appt);
+
+/** Removes EVENT with ical_uid from ical file
+ *  @param ical_uid pointer to ical_uid to be deleted
+ *  @returns: TRUE is successfull, FALSE if failed
+ */
+gboolean xfical_appt_del (gchar *ical_id);
 
 /** Read next EVENT/TODO/JOURNAL component on the specified date from ical
  *  datafile.
@@ -163,23 +253,23 @@ xfical_appt *xfical_appt_get_next_on_day (GDateTime *gdt, gboolean first,
 xfical_appt *xfical_appt_get_next_with_string (const gchar *str, gboolean first,
                                                const gchar *file_type);
 
-void xfical_get_each_app_within_time (GDateTime *a_day, int days
+void xfical_get_each_app_within_time (GDateTime *a_day, gint days
         , xfical_type type, const gchar *file_type , GList **data);
 
 void xfical_mark_calendar(GtkCalendar *gtkcal);
-void xfical_mark_calendar_recur(GtkCalendar *gtkcal, xfical_appt *appt);
+void xfical_mark_calendar_recur(GtkCalendar *gtkcal, const xfical_appt *appt);
 
 void xfical_alarm_build_list(gboolean first_list_today);
 
-int xfical_compare_times(xfical_appt *appt);
+gint xfical_compare_times (xfical_appt *appt);
 #ifdef HAVE_ARCHIVE
 gboolean xfical_archive_open(void);
 void xfical_archive_close(void);
 gboolean xfical_archive(void);
 gboolean xfical_unarchive(void);
-gboolean xfical_unarchive_uid(char *uid);
+gboolean xfical_unarchive_uid (const gchar *uid);
 #endif
 
-gboolean xfical_file_check(gchar *file_name);
+gboolean xfical_file_check (const gchar *file_name);
 
 #endif /* !__ICAL_CODE_H__ */
