@@ -1,5 +1,5 @@
-/*      Orage - Calendar and alarm handler
- *
+/*
+ * Copyright (c) 2021-2023 Erkki Moorits
  * Copyright (c) 2005-2013 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2003-2005 Mickael Graf (korbinus at xfce.org)
  *
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the 
-       Free Software Foundation
-       51 Franklin Street, 5th Floor
-       Boston, MA 02110-1301 USA
-
+ * along with this program; if not, write to the
+ *     Free Software Foundation
+ *     51 Franklin Street, 5th Floor
+ *     Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,13 +38,12 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "orage-i18n.h"
+#include "orage-window.h"
 #include "functions.h"
-#include "mainbox.h"
 #include "interface.h"
 #include "ical-code.h"
 #include "parameters.h"
 
-
 enum {
     DRAG_TARGET_URILIST = 0
    ,DRAG_TARGET_STRING
@@ -72,6 +70,7 @@ static void refresh_foreign_files(intf_win *intf_w, const gboolean first);
 gboolean orage_external_update_check (G_GNUC_UNUSED gpointer user_data)
 {
     struct stat s;
+    OrageApplication *app;
     gint i;
     gboolean external_changes_present = FALSE;
 
@@ -108,7 +107,8 @@ gboolean orage_external_update_check (G_GNUC_UNUSED gpointer user_data)
         g_message ("Refreshing alarms and calendar due to external file update");
         xfical_file_close_force();
         xfical_alarm_build_list(FALSE);
-        orage_mark_appointments();
+        app = ORAGE_APPLICATION (g_application_get_default ());
+        orage_mark_appointments (ORAGE_WINDOW (orage_application_get_window (app)));
     }
 
     return(TRUE); /* keep running */
@@ -306,8 +306,8 @@ static void orage_file_open_button_clicked (G_GNUC_UNUSED GtkButton *button
     gchar *rcfile, *rcdir;
     gchar *s;
 
-    rcdir = g_path_get_dirname((const gchar *)g_par.orage_file);
-    rcfile = g_path_get_basename((const gchar *)g_par.orage_file);
+    rcdir = g_path_get_dirname (g_par.orage_file);
+    rcfile = g_path_get_basename (g_par.orage_file);
     f_chooser = orage_file_chooser(intf_w->main_window, TRUE
             , g_par.orage_file, rcdir, rcfile);
     g_free(rcdir);
@@ -436,8 +436,10 @@ static void on_unarchive_button_clicked_cb (G_GNUC_UNUSED GtkButton *button,
 
 gboolean orage_import_file (const gchar *entry_filename)
 {
+    OrageApplication *app;
     if (xfical_import_file(entry_filename)) {
-        orage_mark_appointments();
+        app = ORAGE_APPLICATION (g_application_get_default ());
+        orage_mark_appointments (ORAGE_WINDOW (orage_application_get_window (app)));
         xfical_alarm_build_list(FALSE);
         return(TRUE);
     }
@@ -445,13 +447,6 @@ gboolean orage_import_file (const gchar *entry_filename)
         return(FALSE);
 }
 
-gboolean orage_export_file (const gchar *entry_filename,
-                            const gint type,
-                            const gchar *uids)
-{
-    return(xfical_export_file(entry_filename, type, uids));
-}
-
 static void imp_save_button_clicked (G_GNUC_UNUSED GtkButton *button,
                                      gpointer user_data)
 {
@@ -512,7 +507,7 @@ static void exp_save_button_clicked (G_GNUC_UNUSED GtkButton *button,
             g_warning ("UNKNOWN select appointment");
         }
 
-        if (orage_export_file(entry_filename, app_count, entry_uids))
+        if (xfical_export_file (entry_filename, app_count, entry_uids))
             g_message ("Export done %s", entry_filename);
         else
             g_warning ("export failed file=%s", entry_filename);
@@ -561,6 +556,7 @@ static void for_open_button_clicked (G_GNUC_UNUSED GtkButton *button,
 static void orage_foreign_file_remove_line(gint del_line)
 {
     int i;
+    OrageApplication *app;
 
     g_free(g_par.foreign_data[del_line].file);
     g_free(g_par.foreign_data[del_line].name);
@@ -575,13 +571,14 @@ static void orage_foreign_file_remove_line(gint del_line)
     g_par.foreign_data[i].name = NULL;
 
     write_parameters();
-    orage_mark_appointments();
+    app = ORAGE_APPLICATION (g_application_get_default ());
+    orage_mark_appointments (ORAGE_WINDOW (orage_application_get_window (app)));
     xfical_alarm_build_list(FALSE);
 }
 
 gboolean orage_foreign_file_remove (const gchar *filename)
 {
-    gint i = 0;
+    gint i;
     gboolean found = FALSE;
 
     if (interface_lock) {
@@ -707,6 +704,7 @@ static gboolean orage_foreign_file_add_internal (const gchar *filename,
                                                  GtkWidget *main_window)
 {
     gint i = 0;
+    OrageApplication *app;
     const gchar *add_failed = _("Foreign file add failed");
 
     if (g_par.foreign_count > 9) {
@@ -783,7 +781,8 @@ static gboolean orage_foreign_file_add_internal (const gchar *filename,
     g_par.foreign_count++;
 
     write_parameters();
-    orage_mark_appointments();
+    app = ORAGE_APPLICATION (g_application_get_default ());
+    orage_mark_appointments (ORAGE_WINDOW (orage_application_get_window (app)));
     xfical_alarm_build_list(FALSE);
     return(TRUE);
 }
@@ -1249,7 +1248,7 @@ static void create_orage_file_tab(intf_win *intf_w)
     label = gtk_label_new(_("Current file:"));
     g_object_set (label, "margin-left", 5, "margin-right", 5, NULL);
     gtk_grid_attach_next_to (GTK_GRID (hbox), label, NULL, GTK_POS_RIGHT, 1, 1);
-    label = gtk_label_new((const gchar *)g_par.orage_file);
+    label = gtk_label_new (g_par.orage_file);
     g_object_set (label, "margin-right", 5,
                          "hexpand", TRUE, "halign", GTK_ALIGN_START,
                          NULL);
@@ -1262,8 +1261,7 @@ static void create_orage_file_tab(intf_win *intf_w)
     g_object_set (label, "margin-left", 5, "margin-right", 5, NULL);
     gtk_grid_attach_next_to (GTK_GRID (hbox), label, NULL, GTK_POS_RIGHT, 1, 1);
     intf_w->orage_file_entry = gtk_entry_new();
-    gtk_entry_set_text(GTK_ENTRY(intf_w->orage_file_entry)
-            , (const gchar *)g_par.orage_file);
+    gtk_entry_set_text (GTK_ENTRY(intf_w->orage_file_entry), g_par.orage_file);
     g_object_set (intf_w->orage_file_entry,
                   "hexpand", TRUE, "halign", GTK_ALIGN_FILL, NULL);
     gtk_grid_attach_next_to (GTK_GRID (hbox), intf_w->orage_file_entry, NULL,
@@ -1495,7 +1493,7 @@ static void create_foreign_file_tab(intf_win *intf_w)
     refresh_foreign_files(intf_w, TRUE);
 }
 
-void orage_external_interface (G_GNUC_UNUSED CalWin *xfcal)
+void orage_external_interface (void)
 {
     intf_win *intf_w = g_new(intf_win, 1);
 
