@@ -1,6 +1,5 @@
-/*      Orage - Calendar and alarm handler
- *
- * Copyright (c) 2023 Erkki Moorits
+/*
+ * Copyright (c) 2023-2024 Erkki Moorits
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,10 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the
-       Free Software Foundation
-       51 Franklin Street, 5th Floor
-       Boston, MA 02110-1301 USA
-
+ *     Free Software Foundation
+ *     51 Franklin Street, 5th Floor
+ *     Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,20 +26,29 @@
 
 #include "reminder.h"
 #include "parameters.h"
+#include "orage-appointment-window.h"
 #include "orage-i18n.h"
 #include "orage-css.h"
-#include "mainbox.h"
+#include "orage-sleep-monitor.h"
+#include "orage-window.h"
 #include "interface.h"
 #include "ical-code.h"
 #include "functions.h"
+#include "event-list.h"
 #include <libxfce4util/libxfce4util.h>
 #include <gtk/gtk.h>
 #include <glib-2.0/gio/gapplication.h>
 
+#ifdef ENABLE_SYNC
+#include "orage-sync-ext-command.h"
+#include "orage-task-runner.h"
+#endif
+
 #define HINT_ADD 'a'
 #define HINT_EXPORT 'x'
 #define HINT_IMPORT 'i'
 #define HINT_REMOVE 'r'
+#define HINT_OPEN 'o'
 
 #define LOGIND_BUS_NAME "org.freedesktop.login1"
 #define LOGIND_IFACE_NAME "org.freedesktop.login1.Manager"
@@ -50,22 +57,23 @@
 struct _OrageApplication
 {
     GtkApplication parent;
-    GDBusConnection *connection;
+    GtkWidget *window;
+    OrageSleepMonitor *sleep_monitor;
+    OrageTaskRunner *sync;
     guint prepare_for_sleep_id;
     gboolean toggle_option;
     gboolean preferences_option;
+    gboolean today_option;
+    gboolean new_appointment_option;
 };
 
 G_DEFINE_TYPE (OrageApplication, orage_application, GTK_TYPE_APPLICATION)
 
 static gboolean window_delete_event_cb (G_GNUC_UNUSED GtkWidget *widget,
                                         G_GNUC_UNUSED GdkEvent *event,
-                                        CalWin *cal)
+                                        OrageApplication *app)
 {
-    if (g_par.close_means_quit)
-        orage_quit ();
-    else
-        gtk_widget_hide (cal->mWindow);
+    orage_application_close (app);
 
     return TRUE;
 }
@@ -80,81 +88,25 @@ static gboolean resuming_after_delay (G_GNUC_UNUSED gpointer user_data)
     return FALSE;
 }
 
-static void resuming_cb (G_GNUC_UNUSED GDBusConnection *connection,
-                         const gchar *sender_name,
-                         const gchar *object_path,
-                         const gchar *interface_name,
-                         const gchar *signal_name,
-                         GVariant *parameters,
-                         G_GNUC_UNUSED gpointer user_data)
+static void woke_up_cb (void)
 {
-    gboolean prepare_for_sleep;
+    g_debug ("received resuming signal");
 
-    g_debug ("%s: sender_name='%s', object_path='%s', %s.%s %s", G_STRFUNC,
-             sender_name, object_path, interface_name,
-             signal_name, g_variant_print (parameters, TRUE));
-
-    if (!g_variant_check_format_string (parameters, "(b)", FALSE))
-    {
-        g_warning ("received incorrect parameter for PrepareForSleep signal");
-        return;
-    }
-
-    g_variant_get (parameters, "(b)", &prepare_for_sleep);
-
-    if (prepare_for_sleep == FALSE)
-    {
-        g_debug ("received resuming signal");
-
-        /* We need this delay to prevent updating tray icon too quickly when the
-         * normal code handles it also.
-         */
-        g_timeout_add_seconds (2, resuming_after_delay, NULL);
-    }
+    /* We need this delay to prevent updating tray icon too quickly when the
+     * normal code handles it also.
+     */
+    g_timeout_add_seconds (2, resuming_after_delay, NULL);
 }
 
 static void resuming_handler_register (OrageApplication *self)
 {
-    GError *error = NULL;
-
-    self->connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
-    if (self->connection == NULL)
+    self->sleep_monitor = orage_sleep_monitor_create ();
+    if (self->sleep_monitor)
     {
-        g_warning ("failed to connect to the D-BUS session bus: %s",
-                   error->message);
-        g_error_free (error);
-        return;
+        g_signal_connect_swapped (G_OBJECT (self->sleep_monitor), "woke-up",
+                                  G_CALLBACK (woke_up_cb), NULL);
+        g_object_ref (G_OBJECT (self->sleep_monitor));
     }
-
-    self->prepare_for_sleep_id = g_dbus_connection_signal_subscribe (
-            self->connection,
-            LOGIND_BUS_NAME,
-            LOGIND_IFACE_NAME,
-            "PrepareForSleep",
-            LOGIND_OBJ_PATH,
-            NULL,
-            G_DBUS_SIGNAL_FLAGS_NONE,
-            resuming_cb,
-            NULL,
-            NULL);
-
-    g_debug ("%s: prepare_for_sleep_id=%d", G_STRFUNC,
-             self->prepare_for_sleep_id);
-}
-
-static void resuming_handler_unregister (OrageApplication *self)
-{
-    g_debug ("%s: prepare_for_sleep_id=%d", G_STRFUNC,
-             self->prepare_for_sleep_id);
-
-    if (self->prepare_for_sleep_id)
-    {
-        g_dbus_connection_signal_unsubscribe (self->connection,
-                                              self->prepare_for_sleep_id);
-    }
-
-    if (self->connection)
-        g_object_unref (self->connection);
 }
 
 static void print_version (void)
@@ -184,86 +136,127 @@ static void print_version (void)
     g_print ("\n");
 }
 
-static void raise_window (void)
+#ifdef ENABLE_SYNC
+static void load_sync_conf (OrageTaskRunner *sync)
 {
-    CalWin *cal = (CalWin *)g_par.xfcal;
+    guint i;
+
+    for (i = 0; i < (guint)g_par.sync_source_count; i++)
+    {
+        orage_task_runner_add (sync, orage_sync_ext_command,
+                               &g_par.sync_conf[i]);
+    }
+}
+#endif
+
+static void raise_window (OrageApplication *self)
+{
+    GtkWindow *window = GTK_WINDOW (self->window);
 
     if (g_par.pos_x || g_par.pos_y)
-        gtk_window_move (GTK_WINDOW (cal->mWindow), g_par.pos_x, g_par.pos_y);
+        gtk_window_move (window, g_par.pos_x, g_par.pos_y);
 
     if (g_par.select_always_today)
-        orage_select_today (GTK_CALENDAR (cal->mCalendar));
+        orage_select_today (orage_window_get_calendar (ORAGE_WINDOW (window)));
 
     if (g_par.set_stick)
-        gtk_window_stick (GTK_WINDOW (cal->mWindow));
+        gtk_window_stick (window);
 
-    gtk_window_set_keep_above (GTK_WINDOW (cal->mWindow), g_par.set_ontop);
-    gtk_window_present (GTK_WINDOW (cal->mWindow));
+    gtk_window_set_keep_above (window, g_par.set_ontop);
+    gtk_window_present (window);
 }
 
 static void orage_application_startup (GApplication *app)
 {
+#ifdef ENABLE_SYNC
+    OrageApplication *self = ORAGE_APPLICATION (app);
+#endif
+
     G_APPLICATION_CLASS (orage_application_parent_class)->startup (app);
-    
-    /* init i18n = nls to use gettext */
-    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
-    register_css_provider ();
+
+    orage_css_set_theme ();
     read_parameters ();
+#ifdef ENABLE_SYNC
+    self->sync = g_object_new (ORAGE_TASK_RUNNER_TYPE, NULL);
+    load_sync_conf (self->sync);
+#endif
+}
+
+static void open_new_appointment_window (void)
+{
+    GtkWidget *appointment_window;
+
+    appointment_window = orage_appointment_window_new_now ();
+    gtk_window_present (GTK_WINDOW (appointment_window));
+}
+
+static void orage_open_today_window (OrageWindow *window)
+{
+    GDateTime *gdt;
+
+    gdt = g_date_time_new_now_local ();
+    orage_select_date (orage_window_get_calendar (window), gdt);
+    g_date_time_unref (gdt);
+    (void)create_el_win (NULL);
 }
 
 static void orage_application_activate (GApplication *app)
 {
     GList *list;
     OrageApplication *self;
+    GtkWidget *window;
+    gboolean hide_main_window;
 
     self = ORAGE_APPLICATION (app);
+    hide_main_window = self->preferences_option
+                     | self->new_appointment_option
+                     | self->today_option;
 
     list = gtk_application_get_windows (GTK_APPLICATION (app));
 
     if (list)
     {
+        /* Orage already running. */
         if (gtk_widget_get_visible (GTK_WIDGET (list->data)) &&
             self->toggle_option)
         {
             write_parameters ();
             gtk_widget_hide (GTK_WIDGET (list->data));
+            hide_main_window = TRUE;
         }
-        else
-            raise_window ();
+        else if (hide_main_window == FALSE)
+            raise_window (self);
+
+        window = self->window;
     }
     else
     {
-        g_par.xfcal = g_new (CalWin, 1);
-
         /* Create the main window */
-        ((CalWin *)g_par.xfcal)->mWindow =
-                gtk_application_window_new (GTK_APPLICATION (app));
+        window = orage_window_new (self);
 
-        g_signal_connect ((gpointer) ((CalWin *)g_par.xfcal)->mWindow,
-                          "delete_event",
-                          G_CALLBACK(window_delete_event_cb),
-                          (gpointer)g_par.xfcal);
+        g_signal_connect (window, "delete_event",
+                          G_CALLBACK (window_delete_event_cb), self);
+
+        self->window = window;
+
+        /* XXX: TODO check if this line is needed for main window (it is copied
+         * from original code).
+         */
+        gtk_window_set_destroy_with_parent (GTK_WINDOW (window), TRUE);
 
-        build_mainWin ();
         set_parameters ();
-        if (g_par.start_visible)
-            gtk_widget_show (((CalWin *)g_par.xfcal)->mWindow);
-        else if (g_par.start_minimized)
-        {
-            gtk_window_iconify (GTK_WINDOW (((CalWin *)g_par.xfcal)->mWindow));
-            gtk_widget_show (((CalWin *)g_par.xfcal)->mWindow);
-        }
-        else
+        if (g_par.start_minimized)
+            gtk_window_iconify (GTK_WINDOW (window));
+        else if (g_par.start_visible == FALSE)
         {
             /* hidden */
-            gtk_widget_realize (((CalWin *)g_par.xfcal)->mWindow);
-            gtk_widget_hide (((CalWin *)g_par.xfcal)->mWindow);
+            gtk_widget_realize (window);
+            hide_main_window = TRUE;
         }
 
         alarm_read ();
         orage_day_change (NULL); /* first day change after we start */
-        mCalendar_month_changed_cb (
-                (GtkCalendar *)((CalWin *)g_par.xfcal)->mCalendar, NULL);
+        orage_window_month_changed (ORAGE_WINDOW (window));
 
         /* start monitoring external file updates */
         g_timeout_add_seconds (30, orage_external_update_check, NULL);
@@ -274,15 +267,34 @@ static void orage_application_activate (GApplication *app)
 
     if (self->preferences_option)
         show_parameters ();
+
+    if (self->new_appointment_option)
+        open_new_appointment_window ();
+
+    if (self->today_option)
+        orage_open_today_window (ORAGE_WINDOW (window));
+
+    if (hide_main_window)
+        gtk_widget_hide (window);
+    else
+        gtk_widget_show (window);
+
+    self->toggle_option = FALSE;
+    self->preferences_option = FALSE;
+    self->new_appointment_option = FALSE;
+    self->today_option = FALSE;
 }
 
 static void orage_application_shutdown (GApplication *app)
 {
-    OrageApplication *self;
+    OrageApplication *self = ORAGE_APPLICATION (app);
 
-    self = ORAGE_APPLICATION (app);
+#ifdef ENABLE_SYNC
+    g_object_unref (self->sync);
+#endif
 
-    resuming_handler_unregister (self);
+    if (self->sleep_monitor)
+        g_object_unref (G_OBJECT (self->sleep_monitor));
 
 #ifdef HAVE_NOTIFY
     orage_notify_uninit ();
@@ -325,12 +337,16 @@ static gint orage_application_command_line (GApplication *app,
     GVariantDict *options;
     gint n_files;
     gint n;
-    OrageApplication *self;
-
-    self = ORAGE_APPLICATION (app);
+    OrageApplication *self = ORAGE_APPLICATION (app);
 
     options = g_application_command_line_get_options_dict (cmdline);
 
+    if (g_variant_dict_contains (options, "today"))
+        self->today_option = TRUE;
+
+    if (g_variant_dict_contains (options, "new-appointment"))
+        self->new_appointment_option = TRUE;
+
     if (g_variant_dict_contains (options, "preferences"))
         self->preferences_option = TRUE;
 
@@ -374,6 +390,19 @@ static gint orage_application_command_line (GApplication *app,
         g_object_unref (file);
     }
 
+    if (g_variant_dict_lookup (options, "import", "^&ay", &file_name))
+    {
+        str_array = g_strsplit (file_name, ":", 2);
+        key[0] = HINT_IMPORT;
+        hint = g_strjoin (":", key, str_array[1], NULL);
+        file = g_application_command_line_create_file_for_arg (cmdline,
+                                                               str_array[0]);
+        g_strfreev (str_array);
+        g_application_open (app, &file, 1, hint);
+        g_free (hint);
+        g_object_unref (file);
+    }
+
     g_variant_dict_lookup (options, G_OPTION_REMAINING, "^a&ay", &filenames);
 
     if (filenames != NULL && (n_files = g_strv_length ((gchar **)filenames)) > 0)
@@ -387,7 +416,7 @@ static gint orage_application_command_line (GApplication *app,
             files[n] = file;
         }
 
-        key[0] = HINT_IMPORT;
+        key[0] = HINT_OPEN;
         g_application_open (app, files, n_files, key);
 
         for (n = 0; n < n_files; n++)
@@ -417,6 +446,10 @@ static void orage_application_open (G_GNUC_UNUSED GApplication *app,
     {
         switch (hint[0])
         {
+            case HINT_OPEN:
+                g_message ("open not yet implemented");
+                break;
+
             case HINT_ADD:
                 file = g_file_get_path (files[i]);
                 hint_array = g_strsplit (hint, ":", 3);
@@ -509,10 +542,10 @@ static void orage_application_class_init (OrageApplicationClass *klass)
     application_class = G_APPLICATION_CLASS (klass);
     application_class->startup = orage_application_startup;
     application_class->activate = orage_application_activate;
-    application_class->shutdown = orage_application_shutdown;
     application_class->open = orage_application_open;
-    application_class->handle_local_options = orage_application_handle_local_options;
     application_class->command_line = orage_application_command_line;
+    application_class->shutdown = orage_application_shutdown;
+    application_class->handle_local_options = orage_application_handle_local_options;
 }
 
 static void orage_application_init (OrageApplication *application)
@@ -520,12 +553,21 @@ static void orage_application_init (OrageApplication *application)
     const GOptionEntry option_entries[] =
     {
         {
-            .long_name = "version",
-            .short_name = 'v',
+            .long_name = "today",
+            .short_name = 'T',
             .flags = G_OPTION_FLAG_NONE,
             .arg = G_OPTION_ARG_NONE,
             .arg_data = NULL,
-            .description = _("Show version of orage"),
+            .description = _("Open today's tasks"),
+            .arg_description = NULL,
+        },
+        {
+            .long_name = "new-appointment",
+            .short_name = 'n',
+            .flags = G_OPTION_FLAG_NONE,
+            .arg = G_OPTION_ARG_NONE,
+            .arg_data = NULL,
+            .description = _("Add new appointment"),
             .arg_description = NULL,
         },
         {
@@ -561,9 +603,18 @@ static void orage_application_init (OrageApplication *application)
             .flags = G_OPTION_FLAG_NONE,
             .arg = G_OPTION_ARG_FILENAME,
             .arg_data = NULL,
-            .description = "Remove a foreign file",
+            .description = _("Remove a foreign file"),
             .arg_description = "<file>",
         },
+        {
+            .long_name = "import",
+            .short_name = 'i',
+            .flags = G_OPTION_FLAG_NONE,
+            .arg = G_OPTION_ARG_FILENAME,
+            .arg_data = NULL,
+            .description = _("Import appointments from file to Orage"),
+            .arg_description = "<file>:[appointment...]",
+        },
         {
             .long_name = "export",
             .short_name = 'e',
@@ -573,6 +624,15 @@ static void orage_application_init (OrageApplication *application)
             .description = _("Export appointments from Orage to file"),
             .arg_description = "<file>:[appointment...]",
         },
+        {
+            .long_name = "version",
+            .short_name = 'v',
+            .flags = G_OPTION_FLAG_NONE,
+            .arg = G_OPTION_ARG_NONE,
+            .arg_data = NULL,
+            .description = _("Show version of Orage"),
+            .arg_description = NULL,
+        },
         {
             .long_name = G_OPTION_REMAINING,
             .short_name = '\0',
@@ -606,3 +666,21 @@ OrageApplication *orage_application_new (void)
                                   G_APPLICATION_HANDLES_OPEN,
                          NULL);
 }
+
+OrageTaskRunner *orage_application_get_sync (OrageApplication *application)
+{
+    return application->sync;
+}
+
+GtkWidget *orage_application_get_window (OrageApplication *application)
+{
+    return application->window;
+}
+
+void orage_application_close (OrageApplication *application)
+{
+    if (g_par.close_means_quit)
+        g_application_quit (G_APPLICATION (application));
+    else
+        gtk_widget_hide (orage_application_get_window (application));
+}
