@@ -1,6 +1,5 @@
-/*      Orage - Calendar and alarm handler
- *
- * Copyright (c) 2022 Erkki Moorits
+/*
+ * Copyright (c) 2022-2023 Erkki Moorits
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,15 +13,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the
-       Free Software Foundation
-       51 Franklin Street, 5th Floor
-       Boston, MA 02110-1301 USA
-
+ *     Free Software Foundation
+ *     51 Franklin Street, 5th Floor
+ *     Boston, MA 02110-1301 USA
  */
 
 #ifndef ORAGE_APPLICATION_H
 #define ORAGE_APPLICATION_H 1
 
+#include "orage-task-runner.h"
 #include <gtk/gtk.h>
 
 G_BEGIN_DECLS
@@ -31,6 +30,9 @@ G_BEGIN_DECLS
 G_DECLARE_FINAL_TYPE (OrageApplication, orage_application, ORAGE, APPLICATION, GtkApplication)
 
 OrageApplication *orage_application_new (void);
+OrageTaskRunner *orage_application_get_sync (OrageApplication *application);
+GtkWidget *orage_application_get_window (OrageApplication *application);
+void orage_application_close (OrageApplication *application);
 
 G_END_DECLS
 
